#include <stdio.h>
#include <stdlib.h>
#include "game.h"
#include "input/input.h"
#include "input/termin.h"

void init_draw()
{
	set_input_mode();
	init_inputdevs();

	puts("\033[?25l");	/* hide cursor */
	puts("\033[0m\033[H\033[2J\033[=18h");
}
	
	
void finish_draw(int sig)
{
	//printf("\033[m\033[H\033[2J\033[?25h");
	printf("\033c");
	restore_input_mode();
	exit(0);
}


void moveto(int x, int y)
{
	printf("\033[%d;%df",y+2,(x+2)*2-1);
}


void set_color(enum Color foreground, enum Color background)
{
	printf("\033[%d;%dm",foreground,background+10);
}


void set_color_style(enum Color foreground, enum Color background, enum Style style)
{
	printf("\033[m\033[%d;%d;%dm",style,foreground,background+10);	
}


//draw a pixel at the current position
void draw_pixel(enum TYPE type)
{	   
	switch(type){
		case UP: set_color_style(Green,Black,Bright); puts("^^"); break;
		case DN: set_color_style(Red,Black,Bright); puts("vv"); break;
		case LF: set_color_style(Magenta,Black,Bright); puts("<-"); break;
		case RT: set_color_style(Cyan,Black,Bright); puts("->"); break;
		case OB: set_color_style(Black,White,Blink); puts("xx"); break;
	}
}


void draw_boundingbox(const struct World * world)
{
	   const int w=world->width;
	   const int h=world->height;
	   
	   //horizontal walls
	   puts("\033[1;47m");
	   moveto(-1,-1);
	   char tmp[10];
	   int i;

	   for(i=-1;i<=w;i++)
	   {
	   	   if(i<0 || i>=w) sprintf(tmp,"  ");
	   	   else sprintf(tmp," %1d",i%10);
	 	   printf("%s",tmp);
	   }
	   
	   moveto(-1,h); //move to the last row
	   
	   for(i=-1;i<=w;i++)
	   {
	   	   if(i<0 || i>=w) sprintf(tmp,"  ");
	   	   else sprintf(tmp," %1d",i%10);
	 	   printf("%s",tmp);
	   }

	   
	   //vertical walls

	   for(i=0;i<h;i++){
 	   	    sprintf(tmp," %1d",i%10);
	  		moveto(-1,i); //move to the first elemet of i-th row
	   		puts(tmp);
	   		
  	   	    sprintf(tmp,"%1d ",i%10);
  	   	    moveto(w,i); //move to the first elemet of i-th row
	   		puts(tmp);
	   }

}

