#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>

#include "game.h"
#include "input/input.h"
#include "input/termin.h"

// handle alarm signal
// this function will move the particle randomly 
// TODO: change this to let the particle follow the vector field
void catch_alarm(int sig)
{
	event.type=AlarmEvent;
	event.handled=0; //set to not handled
	event.delta_x=0;
	event.delta_y=0;
	
	switch(rand()%4){
		case 1: event.delta_y=+1; break; //up
		case 2: event.delta_y=-1; break; //down
		case 3: event.delta_x=+1; break; //right
		case 0: event.delta_x=-1; break; //left
	}
	
    signal(SIGALRM, catch_alarm);
	alarm (1);
}

//
// this function listen to the keyboard event
// and handle the keyboard event accordingly
//
void key_control(struct World * world)
{			
	  if(waitinput(0,1)) //input is triggered
	  {
		  event.type=KeyboardEvent;
		  event.handled=0; //set to not handled
		  event.delta_x=0;
		  event.delta_y=0;
		  int key = getkeypress_block(SINGLE_PL);
				  
		  switch (key) {
			  case MVLEFT:
				  event.delta_x=-1;
				  break;
			  case MVRIGHT:
				  event.delta_x= 1;
				  break;
			  case MVUP:
				  event.delta_y=-1;
				  break;
			  case MVDOWN:
				  event.delta_y= 1;
				  break;
			  case 'q': raise(SIGINT);
		  }
	  }
}

int main(int argc, char ** argv) 
{
	
	if(argc!=3){ fprintf(stderr,"Usage: %s widith height\n",argv[0]); return 1; }
	
	   int i, j;
	   
	   //init the game
	   world.width=atoi(argv[1]);
	   world.height=atoi(argv[2]);
	   world.px=0;
	   world.py=0;
	   event.delta_x=0;
	   event.delta_y=0;
	   event.handled=0;
	   
	   init_draw();
	   draw_boundingbox(&world);
	   signal(SIGINT,  finish_draw); //for terminating the game
	   signal(SIGALRM, catch_alarm); //for animation
	   alarm(1); //this will setup the animation cycle
    
	   //create a random environment
	   //TODO: you should create the environment based on the input file
	   srand(time(NULL));
	   for(i=0;i<world.width;i++){
		   for(j=0;j<world.height;j++){
	   		   moveto(i,j);
			   draw_pixel(UP+rand()%5); 
			}
	   }


	  //game loop
	  char tmp[10];
	  while(1)
	  { 
	  		if(event.handled==0) //not handled
			{
				//erase the last particle position
				moveto(world.px,world.py);
				set_color(Yellow,Yellow);
				puts("  "); //two space, clear the last particle position
				
				//get new position for particle
				world.px+=event.delta_x;
				world.py+=event.delta_y;				

				
				//make sure that the particle is not out of bound
				if(world.py<0) world.py=0;
				if(world.py>world.height-1) world.py=world.height-1;
				if(world.px<0) world.px=0;
				if(world.px>world.width-1) world.px=world.width-1;
				
				//move to the new place
				moveto(world.px,world.py);
				set_color_style(White,Red,Blink);
				puts("@@");
				fflush(stdout);
				
				//marked as handled
				event.handled=1;
				event.delta_x=0;
				event.delta_y=0;
			}
			
			//get new keyboard evnet
			key_control(&world);
	  }
       
      return 0;
}


